@@ -12,10 +12,20 @@
 #include <stdlib.h>
 #include <netinet/in.h> /* For htons */
 #include <libnetfilter_conntrack/libnetfilter_conntrack.h>
-#include <libnetfilter_conntrack/libnetfilter_conntrack_udp.h>
 
 #include "conntrack.h"
 
+enum {
+	CT_UDP_ORIG_SPORT =	(1 << 0),
+	CT_UDP_ORIG_DPORT =	(1 << 1),
+	CT_UDP_REPL_SPORT =	(1 << 2),
+	CT_UDP_REPL_DPORT =	(1 << 3),
+	CT_UDP_MASK_SPORT =	(1 << 4),
+	CT_UDP_MASK_DPORT =	(1 << 5),
+	CT_UDP_EXPTUPLE_SPORT =	(1 << 6),
+	CT_UDP_EXPTUPLE_DPORT =	(1 << 7)
+};
+
 static struct option opts[] = {
 	{"orig-port-src", 1, 0, '1'},
 	{"sport", 1, 0, '1'},
@@ -54,10 +64,10 @@ static char udp_commands_v_options[NUMBER_OF_CMD][UDP_NUMBER_OF_OPT] =
 {
 		/* 1 2 3 4 5 6 7 8 */
 /*CT_LIST*/	  {2,2,2,2,0,0,0,0},
-/*CT_CREATE*/     {2,2,2,2,0,0,0,0},
+/*CT_CREATE*/     {3,3,3,3,0,0,0,0},
 /*CT_UPDATE*/     {2,2,2,2,0,0,0,0},
 /*CT_DELETE*/     {2,2,2,2,0,0,0,0},
-/*CT_GET*/        {2,2,2,2,0,0,0,0},
+/*CT_GET*/        {3,3,3,3,0,0,0,0},
 /*CT_FLUSH*/      {0,0,0,0,0,0,0,0},
 /*CT_EVENT*/      {2,2,2,2,0,0,0,0},
 /*CT_VERSION*/    {0,0,0,0,0,0,0,0},
@@ -82,37 +92,37 @@ static int parse_options(char c,
 			port = htons(atoi(optarg));
 			nfct_set_attr_u16(ct, ATTR_ORIG_PORT_SRC, port);
 			nfct_set_attr_u8(ct, ATTR_ORIG_L4PROTO, IPPROTO_UDP);
-			*flags |= UDP_ORIG_SPORT;
+			*flags |= CT_UDP_ORIG_SPORT;
 			break;
 		case '2':
 			port = htons(atoi(optarg));
 			nfct_set_attr_u16(ct, ATTR_ORIG_PORT_DST, port);
 			nfct_set_attr_u8(ct, ATTR_ORIG_L4PROTO, IPPROTO_UDP);
-			*flags |= UDP_ORIG_DPORT;
+			*flags |= CT_UDP_ORIG_DPORT;
 			break;
 		case '3':
 			port = htons(atoi(optarg));
 			nfct_set_attr_u16(ct, ATTR_REPL_PORT_SRC, port);
 			nfct_set_attr_u8(ct, ATTR_REPL_L4PROTO, IPPROTO_UDP);
-			*flags |= UDP_REPL_SPORT;
+			*flags |= CT_UDP_REPL_SPORT;
 			break;
 		case '4':
 			port = htons(atoi(optarg));
 			nfct_set_attr_u16(ct, ATTR_REPL_PORT_DST, port);
 			nfct_set_attr_u8(ct, ATTR_REPL_L4PROTO, IPPROTO_UDP);
-			*flags |= UDP_REPL_DPORT;
+			*flags |= CT_UDP_REPL_DPORT;
 			break;
 		case '5':
 			port = htons(atoi(optarg));
 			nfct_set_attr_u16(mask, ATTR_ORIG_PORT_SRC, port);
 			nfct_set_attr_u8(mask, ATTR_ORIG_L4PROTO, IPPROTO_UDP);
-			*flags |= UDP_MASK_SPORT;
+			*flags |= CT_UDP_MASK_SPORT;
 			break;
 		case '6':
 			port = htons(atoi(optarg));
 			nfct_set_attr_u16(mask, ATTR_ORIG_PORT_DST, port);
 			nfct_set_attr_u8(mask, ATTR_ORIG_L4PROTO, IPPROTO_UDP);
-			*flags |= UDP_MASK_DPORT;
+			*flags |= CT_UDP_MASK_DPORT;
 			break;
 		case '7':
 			port = htons(atoi(optarg));
@@ -120,7 +130,7 @@ static int parse_options(char c,
 			nfct_set_attr_u8(exptuple,
 					 ATTR_ORIG_L4PROTO,
 					 IPPROTO_UDP);
-			*flags |= UDP_EXPTUPLE_SPORT;
+			*flags |= CT_UDP_EXPTUPLE_SPORT;
 			break;
 		case '8':
 			port = htons(atoi(optarg));
@@ -128,25 +138,42 @@ static int parse_options(char c,
 			nfct_set_attr_u8(exptuple,
 					 ATTR_ORIG_L4PROTO,
 					 IPPROTO_UDP);
-			*flags |= UDP_EXPTUPLE_DPORT;
+			*flags |= CT_UDP_EXPTUPLE_DPORT;
 			break;
 	}
 	return 1;
 }
 
+#define UDP_VALID_FLAGS_MAX   2
+static unsigned int udp_valid_flags[UDP_VALID_FLAGS_MAX] = {
+       CT_UDP_ORIG_SPORT | CT_UDP_ORIG_DPORT,
+       CT_UDP_REPL_SPORT | CT_UDP_REPL_DPORT,
+};
+
 static void final_check(unsigned int flags,
 		        unsigned int cmd,
 		        struct nf_conntrack *ct)
 {
-	if ((1 << cmd) & (CT_CREATE|CT_GET) &&
-	    !((flags & UDP_ORIG_SPORT && flags & UDP_ORIG_DPORT) ||
-	      (flags & UDP_REPL_SPORT && flags & UDP_REPL_DPORT)))
-		exit_error(PARAMETER_PROBLEM, "missing ports");
+	int ret, partial;
 
-	generic_opt_check(flags, 
-			  UDP_NUMBER_OF_OPT,
-			  udp_commands_v_options[cmd],
-			  udp_optflags);
+	ret = generic_opt_check(flags, UDP_NUMBER_OF_OPT,
+				udp_commands_v_options[cmd], udp_optflags,
+				udp_valid_flags, UDP_VALID_FLAGS_MAX, &partial);
+	if (!ret) {
+		switch(partial) {
+		case -1:
+		case 0:
+			exit_error(PARAMETER_PROBLEM, "you have to specify "
+						      "`--sport' and "
+						      "`--dport'");
+			break;
+		case 1:
+			exit_error(PARAMETER_PROBLEM, "you have to specify "
+						      "`--reply-src-port' and "
+						      "`--reply-dst-port'");
+			break;
+		}
+	}
 }
 
 static struct ctproto_handler udp = {
